#!/usr/bin/python3

import os
import sys
from copy import deepcopy

from dpugen.confbase import ConfBase
from dpugen.confutils import common_main


class AclRules(ConfBase):

    def __init__(self, params={}):
        super().__init__(params)
        self.num_yields = 0

    def items(self):
        print('  Generating %s ...' % os.path.basename(__file__), file=sys.stderr)
        p = self.params
        cp = self.cooked_params

        for eni_index, eni in enumerate(range(p.ENI_START, p.ENI_START + p.ENI_COUNT)):
            local_ip = cp.IP_L_START + eni_index * cp.IP_STEP_ENI
            l_ip_ac = deepcopy(str(local_ip)+'/32')

            for stage_in_index in range(p.ACL_NSG_COUNT):
                nsg_index = stage_in_index + 1
                table_id = eni * 1000 + stage_in_index

                for ip_index in range(0, p.ACL_RULES_NSG, 2):
                    # print("        %d" % ip_index)
                    remote_ip_a = cp.IP_R_START + eni_index * cp.IP_STEP_ENI + stage_in_index * cp.IP_STEP_NSG + ip_index * cp.IP_STEP_ACL

                    ip_list_a = [str(remote_ip_a + expanded_index * cp.IP_STEPE) +
                                 '/32' for expanded_index in range(0, p.IP_PER_ACL_RULE)]

                    # allow
                    self.num_yields += 1
                    yield {
                        'name': 'dash_acl_%d_rule_%d' % (table_id, ip_index),
                        'op': 'create',
                        'type': 'SAI_OBJECT_TYPE_DASH_ACL_RULE',
                        'attributes': [
                            'SAI_DASH_ACL_RULE_ATTR_DASH_ACL_GROUP_ID', f'$in_acl_group_#eni{eni}nsg{nsg_index}',
                            'SAI_DASH_ACL_RULE_ATTR_PRIORITY',          '%d' % ip_index,
                            'SAI_DASH_ACL_RULE_ATTR_ACTION',            'SAI_DASH_ACL_RULE_ACTION_PERMIT',
                            'SAI_DASH_ACL_RULE_ATTR_SIP',               ','.join(ip_list_a[:]),
                            'SAI_DASH_ACL_RULE_ATTR_DIP',               l_ip_ac,
                            # 'SAI_DASH_ACL_RULE_ATTR_PROTOCOL',          'sai_u8_list_t',
                            # 'SAI_DASH_ACL_RULE_ATTR_SRC_PORT',          'sai_u16_range_list_t',
                            # 'SAI_DASH_ACL_RULE_ATTR_DST_PORT',          'sai_u16_range_list_t',
                        ]
                    }

                    remote_ip_d = remote_ip_a - cp.IP_STEP1

                    ip_list_d = [str(remote_ip_d + expanded_index * cp.IP_STEPE) + '/32' for expanded_index in range(0, p.IP_PER_ACL_RULE)]

                    # denny
                    self.num_yields += 1
                    yield {
                        'name': 'dash_acl_%d_rule_%d' % (table_id, ip_index+1),
                        'op': 'create',
                        'type': 'SAI_OBJECT_TYPE_DASH_ACL_RULE',
                        'attributes': [
                            'SAI_DASH_ACL_RULE_ATTR_DASH_ACL_GROUP_ID', f'$in_acl_group_#eni{eni}nsg{nsg_index}',
                            'SAI_DASH_ACL_RULE_ATTR_PRIORITY',          f'{ip_index+1}',
                            'SAI_DASH_ACL_RULE_ATTR_ACTION',            'SAI_DASH_ACL_RULE_ACTION_DENY',
                            'SAI_DASH_ACL_RULE_ATTR_SIP',               ','.join(ip_list_d[:]),
                            'SAI_DASH_ACL_RULE_ATTR_DIP',               l_ip_ac,
                            # 'SAI_DASH_ACL_RULE_ATTR_PROTOCOL',          'sai_u8_list_t',
                            # 'SAI_DASH_ACL_RULE_ATTR_SRC_PORT',          'sai_u16_range_list_t',
                            # 'SAI_DASH_ACL_RULE_ATTR_DST_PORT',          'sai_u16_range_list_t',
                        ]
                    }

                # add as last rule in last table from ingress and egress an allow rule for all the ip's from egress and ingress
                if (stage_in_index % p.ACL_NSG_COUNT) == 4:
                    all_ips_stage1 = cp.IP_R_START + eni_index * cp.IP_STEP_ENI + stage_in_index * 4 * cp.IP_STEP_NSG
                    all_ips_stage2 = all_ips_stage1 + 1 * 4 * cp.IP_STEP_NSG
                    all_ips_stage3 = all_ips_stage1 + 2 * 4 * cp.IP_STEP_NSG
                    all_ips_stage4 = all_ips_stage1 + 3 * 4 * cp.IP_STEP_NSG
                    all_ips_stage5 = all_ips_stage1 + 4 * 4 * cp.IP_STEP_NSG
                    ip_list_all = [
                        str(all_ips_stage1)+'/14',
                        str(all_ips_stage2)+'/14',
                        str(all_ips_stage3)+'/14',
                        str(all_ips_stage4)+'/14',
                        str(all_ips_stage5)+'/14',
                    ]

                    # allow
                    self.num_yields += 1
                    yield {
                        'name': 'dash_acl_%d_rule_%d' % (table_id, ip_index+2),
                        'op': 'create',
                        'type': 'SAI_OBJECT_TYPE_DASH_ACL_RULE',
                        'attributes': [
                            'SAI_DASH_ACL_RULE_ATTR_DASH_ACL_GROUP_ID', f'$in_acl_group_#eni{eni}nsg{nsg_index}',
                            'SAI_DASH_ACL_RULE_ATTR_PRIORITY',          f'{ip_index+2}',
                            'SAI_DASH_ACL_RULE_ATTR_ACTION',            'SAI_DASH_ACL_RULE_ACTION_PERMIT',
                            'SAI_DASH_ACL_RULE_ATTR_SIP',               ','.join(ip_list_all[:]),
                            'SAI_DASH_ACL_RULE_ATTR_DIP',               l_ip_ac,
                            # 'SAI_DASH_ACL_RULE_ATTR_PROTOCOL',          'sai_u8_list_t',
                            # 'SAI_DASH_ACL_RULE_ATTR_SRC_PORT',          'sai_u16_range_list_t',
                            # 'SAI_DASH_ACL_RULE_ATTR_DST_PORT',          'sai_u16_range_list_t',
                        ]
                    }

            for stage_out_index in range(p.ACL_NSG_COUNT):
                table_id = eni * 1000 + 500 + stage_out_index

                for ip_index in range(0, p.ACL_RULES_NSG, 2):
                    # print("        %d" % ip_index)
                    remote_ip_a = cp.IP_R_START + eni_index * cp.IP_STEP_ENI + \
                        (p.ACL_NSG_COUNT + stage_in_index) * cp.IP_STEP_NSG + ip_index * cp.IP_STEP_ACL

                    ip_list_a = [str(remote_ip_a + expanded_index * cp.IP_STEPE) +
                                 '/32' for expanded_index in range(0, p.IP_PER_ACL_RULE)]

                    # allow
                    self.num_yields += 1
                    yield {
                        'name': 'dash_acl_%d_rule_%d' % (table_id, ip_index),
                        'op': 'create',
                        'type': 'SAI_OBJECT_TYPE_DASH_ACL_RULE',
                        'attributes': [
                            'SAI_DASH_ACL_RULE_ATTR_DASH_ACL_GROUP_ID', f'$out_acl_group_#eni{eni}nsg{nsg_index}',
                            'SAI_DASH_ACL_RULE_ATTR_PRIORITY',          f'{ip_index}',
                            'SAI_DASH_ACL_RULE_ATTR_ACTION',            'SAI_DASH_ACL_RULE_ACTION_PERMIT',
                            'SAI_DASH_ACL_RULE_ATTR_SIP',               l_ip_ac,
                            'SAI_DASH_ACL_RULE_ATTR_DIP',               ','.join(ip_list_a[:]),
                            # 'SAI_DASH_ACL_RULE_ATTR_PROTOCOL',          'sai_u8_list_t',
                            # 'SAI_DASH_ACL_RULE_ATTR_SRC_PORT',          'sai_u16_range_list_t',
                            # 'SAI_DASH_ACL_RULE_ATTR_DST_PORT',          'sai_u16_range_list_t',
                        ]
                    }

                    remote_ip_d = remote_ip_a - cp.IP_STEP1

                    ip_list_d = [str(remote_ip_d + expanded_index * cp.IP_STEPE) + '/32' for expanded_index in range(0, p.IP_PER_ACL_RULE)]

                    # denny
                    self.num_yields += 1
                    yield {
                        'name': 'dash_acl_%d_rule_%d' % (table_id, ip_index+1),
                        'op': 'create',
                        'type': 'SAI_OBJECT_TYPE_DASH_ACL_RULE',
                        'attributes': [
                            'SAI_DASH_ACL_RULE_ATTR_DASH_ACL_GROUP_ID', f'$out_acl_group_#eni{eni}nsg{nsg_index}',
                            'SAI_DASH_ACL_RULE_ATTR_PRIORITY',          f'{ip_index+1}',
                            'SAI_DASH_ACL_RULE_ATTR_ACTION',            'SAI_DASH_ACL_RULE_ACTION_DENY',
                            'SAI_DASH_ACL_RULE_ATTR_SIP',               l_ip_ac,
                            'SAI_DASH_ACL_RULE_ATTR_DIP',               ','.join(ip_list_d[:]),
                            # 'SAI_DASH_ACL_RULE_ATTR_PROTOCOL',          'sai_u8_list_t',
                            # 'SAI_DASH_ACL_RULE_ATTR_SRC_PORT',          'sai_u16_range_list_t',
                            # 'SAI_DASH_ACL_RULE_ATTR_DST_PORT',          'sai_u16_range_list_t',
                        ]
                    }

                # add as last rule in last table from ingress and egress an allow rule for all the ip's from egress and ingress
                if (stage_out_index % p.ACL_NSG_COUNT) == 4:
                    all_ips_stage1 = cp.IP_R_START + eni_index * cp.IP_STEP_ENI
                    all_ips_stage2 = all_ips_stage1 + 1 * 4 * cp.IP_STEP_NSG
                    all_ips_stage3 = all_ips_stage1 + 2 * 4 * cp.IP_STEP_NSG
                    all_ips_stage4 = all_ips_stage1 + 3 * 4 * cp.IP_STEP_NSG
                    all_ips_stage5 = all_ips_stage1 + 4 * 4 * cp.IP_STEP_NSG
                    ip_list_all = [
                        str(all_ips_stage1)+'/14',
                        str(all_ips_stage2)+'/14',
                        str(all_ips_stage3)+'/14',
                        str(all_ips_stage4)+'/14',
                        str(all_ips_stage5)+'/14',
                    ]

                    # allow
                    self.num_yields += 1
                    yield {
                        'name': 'dash_acl_%d_rule_%d' % (table_id, ip_index+2),
                        'op': 'create',
                        'type': 'SAI_OBJECT_TYPE_DASH_ACL_RULE',
                        'attributes': [
                            'SAI_DASH_ACL_RULE_ATTR_DASH_ACL_GROUP_ID', f'$out_acl_group_#eni{eni}nsg{nsg_index}',
                            'SAI_DASH_ACL_RULE_ATTR_PRIORITY',          f'{ip_index+2}',
                            'SAI_DASH_ACL_RULE_ATTR_ACTION',            'SAI_DASH_ACL_RULE_ACTION_PERMIT',
                            'SAI_DASH_ACL_RULE_ATTR_SIP',               l_ip_ac,
                            'SAI_DASH_ACL_RULE_ATTR_DIP',               ','.join(ip_list_all[:]),
                            # 'SAI_DASH_ACL_RULE_ATTR_PROTOCOL',          'sai_u8_list_t',
                            # 'SAI_DASH_ACL_RULE_ATTR_SRC_PORT',          'sai_u16_range_list_t',
                            # 'SAI_DASH_ACL_RULE_ATTR_DST_PORT',          'sai_u16_range_list_t',
                        ]
                    }


if __name__ == '__main__':
    conf = AclRules()
    common_main(conf)



#!/usr/bin/python3

import os
import sys
from copy import deepcopy

from dpugen.confbase import ConfBase
from dpugen.confutils import common_main


class AclRules(ConfBase):

    def __init__(self, params={}):
        super().__init__(params)
        self.num_yields = 0

    def items(self):
        print('  Generating %s ...' % os.path.basename(__file__), file=sys.stderr)
        p = self.params
        cp = self.cooked_params

        for eni_index, eni in enumerate(range(p.ENI_START, p.ENI_START + p.ENI_COUNT * p.ENI_STEP, p.ENI_STEP)):  # Per ENI (64)
            local_ip = cp.IP_L_START + eni_index * cp.IP_STEP_ENI
            l_ip_ac = deepcopy(str(local_ip)+'/32')
            for stage_in_index in range(p.ACL_NSG_COUNT):  # Per inbound group
                table_id = eni * 1000 + stage_in_index
                for ip_index in range(0, p.ACL_RULES_NSG, 2):  # Per even ACL rule
                    remote_ip_a = cp.IP_R_START + (eni_index * cp.IP_STEP_ENI) + (stage_in_index * cp.IP_STEP_NSG) + ((ip_index // 2) * cp.IP_STEP_ACL)
                    ip_list_a = [str(remote_ip_a + expanded_index * cp.IP_STEPE) + '/32' for expanded_index in range(0, p.IP_PER_ACL_RULE)]

                    # Allow
                    self.num_yields += 1
                    yield {
                        'DASH_ACL_RULE_TABLE:%d:rule%d' % (table_id, ip_index): {
                            'priority': ip_index,
                            'action': 'allow',
                            'terminating': 'true',
                            'src_addr': ','.join(ip_list_a[:]),
                            'dst_addr': l_ip_ac
                        },
                        'OP': 'SET'
                    }

                    remote_ip_d = remote_ip_a - cp.IP_STEP1
                    ip_list_d = [str(remote_ip_d + expanded_index * cp.IP_STEPE) + '/32' for expanded_index in range(0, p.IP_PER_ACL_RULE)]

                    if ip_index == (p.ACL_RULES_NSG - 2) and (stage_in_index % p.ACL_NSG_COUNT) == 4:
                        break  # Skip the very last rule on the last stage to have exactly 1000 rules

                    # Deny
                    self.num_yields += 1
                    yield {
                        'DASH_ACL_RULE_TABLE:%d:rule%d' % (table_id, ip_index + 1): {
                            'priority': ip_index + 1,
                            'action': 'deny',
                            'terminating': 'true',
                            'src_addr': ','.join(ip_list_d[:]),
                            'dst_addr': l_ip_ac
                        },
                        'OP': 'SET'
                    }

                # add as last rule in last table from ingress and egress an allow rule for all the IPs from egress and ingress
                if (stage_in_index % p.ACL_NSG_COUNT) == 4:
                    all_ips_stage1 = cp.IP_L_START + eni_index * cp.IP_STEP_ENI + stage_in_index * 4 * cp.IP_STEP_NSG
                    all_ips_stage2 = all_ips_stage1 + 1 * 4 * cp.IP_STEP_NSG
                    all_ips_stage3 = all_ips_stage1 + 2 * 4 * cp.IP_STEP_NSG
                    all_ips_stage4 = all_ips_stage1 + 3 * 4 * cp.IP_STEP_NSG
                    all_ips_stage5 = all_ips_stage1 + 4 * 4 * cp.IP_STEP_NSG
                    ip_list_all = [
                        str(all_ips_stage1)+'/14',
                        str(all_ips_stage2)+'/14',
                        str(all_ips_stage3)+'/14',
                        str(all_ips_stage4)+'/14',
                        str(all_ips_stage5)+'/14',
                    ]

                    # Allow
                    self.num_yields += 1
                    yield {
                        'DASH_ACL_RULE_TABLE:%d:rule%d' % (table_id, ip_index + 1): {
                            'priority': ip_index + 1,
                            'action': 'allow',
                            'terminating': 'true',
                            'src_addr': ','.join(ip_list_all[:]),
                            'dst_addr': l_ip_ac
                        },
                        'OP': 'SET'
                    }

            for stage_out_index in range(p.ACL_NSG_COUNT):
                table_id = eni * 1000 + 500 + stage_out_index
                for ip_index in range(0, p.ACL_RULES_NSG, 2):
                    remote_ip_a = cp.IP_R_START + (eni_index * cp.IP_STEP_ENI) + (stage_out_index * cp.IP_STEP_NSG) + ((ip_index // 2) * cp.IP_STEP_ACL)
                    ip_list_a = [str(remote_ip_a + expanded_index * cp.IP_STEPE) + '/32' for expanded_index in range(0, p.IP_PER_ACL_RULE)]

                    # Allow
                    self.num_yields += 1
                    yield {
                        'DASH_ACL_RULE_TABLE:%d:rule%d' % (table_id, ip_index): {
                            'priority': ip_index,
                            'action': 'allow',
                            'terminating': 'true',
                            'src_addr': l_ip_ac,
                            'dst_addr': ','.join(ip_list_a[:])
                        },
                        'OP': 'SET'
                    }

                    remote_ip_d = remote_ip_a - cp.IP_STEP1
                    ip_list_d = [str(remote_ip_d + expanded_index * cp.IP_STEPE) + '/32' for expanded_index in range(0, p.IP_PER_ACL_RULE)]

                    if ip_index == (p.ACL_RULES_NSG - 2) and (stage_out_index % p.ACL_NSG_COUNT) == 4:
                        break  # Skip the very last rule on the last stage to have exactly 1000 rules

                    # Deny
                    self.num_yields += 1
                    yield {
                        'DASH_ACL_RULE_TABLE:%d:rule%d' % (table_id, ip_index + 1): {
                            'priority': ip_index + 1,
                            'action': 'deny',
                            'terminating': 'true',
                            'src_addr': l_ip_ac,
                            'dst_addr': ','.join(ip_list_d[:])
                        },
                        'OP': 'SET'
                    }

                # add as last rule in last table from ingress and egress an allow rule for all the IPs from egress and ingress
                if (stage_out_index % p.ACL_NSG_COUNT) == 4:
                    all_ips_stage1 = cp.IP_R_START + eni_index * cp.IP_STEP_ENI
                    all_ips_stage2 = all_ips_stage1 + 1 * 4 * cp.IP_STEP_NSG
                    all_ips_stage3 = all_ips_stage1 + 2 * 4 * cp.IP_STEP_NSG
                    all_ips_stage4 = all_ips_stage1 + 3 * 4 * cp.IP_STEP_NSG
                    all_ips_stage5 = all_ips_stage1 + 4 * 4 * cp.IP_STEP_NSG
                    ip_list_all = [
                        str(all_ips_stage1) + '/14',
                        str(all_ips_stage2) + '/14',
                        str(all_ips_stage3) + '/14',
                        str(all_ips_stage4) + '/14',
                        str(all_ips_stage5) + '/14',
                    ]

                    # Allow
                    self.num_yields += 1
                    yield {
                        'DASH_ACL_RULE_TABLE:%d:rule%d' % (table_id, ip_index + 1): {
                            'priority': ip_index + 1,
                            'action': 'allow',
                            'terminating': 'true',
                            'src_addr': l_ip_ac,
                            'dst_addr': ','.join(ip_list_all[:])
                        },
                        'OP': 'SET'
                    }


if __name__ == '__main__':
    conf = AclRules()
    common_main(conf)



#!/usr/bin/python3

import math
import os
import sys

from dpugen.confbase import (
    ConfBase,
    ipa
)
from dpugen.confutils import common_main


class RouteTables(ConfBase):

    def __init__(self, params={}):
        super().__init__(params)
        self.num_yields = 0

    def items(self):
        print('  Generating %s ...' % os.path.basename(__file__), file=sys.stderr)
        p = self.params
        cp = self.cooked_params
        nr_of_routes_prefixes = int(math.log(p.IP_ROUTE_DIVIDER_PER_ACL_RULE, 2))
        for eni_index, eni in enumerate(range(p.ENI_START, p.ENI_START + p.ENI_COUNT * p.ENI_STEP, p.ENI_STEP)):  # Per ENI (64)
            vtep_eni = str(ipa(p.PAL) + int(ipa(p.IP_STEP1)) * eni_index)
            added_route_count = 0
            for table_index in range(p.ACL_NSG_COUNT):  # Per outbound group (5)
                for acl_index in range(p.ACL_RULES_NSG // 2):  # Per even rule (1000 / 2)
                    remote_ip = cp.IP_R_START + (eni_index * cp.IP_STEP_ENI) + (table_index * cp.IP_STEP_NSG) + (acl_index * cp.IP_STEP_ACL)
                    no_of_route_groups = p.IP_PER_ACL_RULE // (p.IP_ROUTE_DIVIDER_PER_ACL_RULE // 2)
                    for ip_index in range(0, no_of_route_groups):
                        ip_prefix = (remote_ip - 1) + (ip_index * p.IP_ROUTE_DIVIDER_PER_ACL_RULE * cp.IP_STEP1)
                        for prefix_index in range(nr_of_routes_prefixes, 0, -1):
                            nr_of_ips = 1 << (prefix_index - 1)
                            mask = 32 - prefix_index + 1
                            if mask == 32:
                                ip_prefix = ip_prefix + 1

                            if (eni % 4) == 1:
                                self.num_yields += 1
                                yield {
                                    'DASH_ROUTE_TABLE:eni-%d:%s/%d' % (eni, ip_prefix, mask): {
                                        'action_type': 'vnet',
                                        'vnet': 'vnet-%d' % (eni + p.ENI_L2R_STEP)
                                    },
                                    'OP': 'SET'
                                }

                            else:
                                # routes that do not have a mac mapping
                                self.num_yields += 1
                                yield {
                                    'DASH_ROUTE_TABLE:eni-%d:%s/%d' % (eni, ip_prefix, mask): {
                                        'action_type': 'vnet_direct',
                                        'vnet': 'vnet-%d' % (eni + p.ENI_L2R_STEP),
                                        'overlay_ip': vtep_eni
                                    },
                                    'OP': 'SET'
                                }

                            added_route_count += 1
                            ip_prefix = ip_prefix + cp.IP_STEP1 * nr_of_ips

                    # TODO: transition between mapped and routed ips

            # TODO: write condition check here to add a default route if no route was added to current ENI'
            if added_route_count == 0:
                remote_ip_prefix = cp.IP_R_START + eni_index * cp.IP_STEP_ENI
                self.num_yields += 1
                yield {
                    'DASH_ROUTE_TABLE:eni-%d:%s/%d' % (eni, remote_ip_prefix, 10): {
                        'action_type': 'vnet',
                        'vnet': 'vnet-%d' % (eni + p.ENI_L2R_STEP)
                    },
                    'OP': 'SET'
                }


if __name__ == '__main__':
    conf = RouteTables()
    common_main(conf)



#!/usr/bin/python3

import os
import sys

from dpugen.confbase import ConfBase, ipa, maca
from dpugen.confutils import common_main


class Mappings(ConfBase):

    def __init__(self, params={}):
        super().__init__(params)
        self.num_yields = 0

    def items(self):
        print('  Generating %s ...' % os.path.basename(__file__), file=sys.stderr)
        p = self.params

        for eni_index, eni in enumerate(range(p.ENI_START, p.ENI_START + p.ENI_COUNT * p.ENI_STEP, p.ENI_STEP)):  # Per ENI
            vtep_remote = str(ipa(p.PAR) + int(ipa(p.IP_STEP1)) * eni_index)
            vtep_eni = str(ipa(p.PAL) + int(ipa(p.IP_STEP1)) * eni_index)
            r_vni_id = eni + p.ENI_L2R_STEP

            # 1 in 4 enis will have all its ips mapped
            if (eni % 4) == 1:
                # mapped IPs
                print(f'    mapped:eni:{eni}', file=sys.stderr)
                for nsg_index in range(p.ACL_NSG_COUNT):  # Per outbound stage
                    for acl_index in range(p.ACL_RULES_NSG // 2):  # Per half of the rules
                        remote_ip_a = ipa(p.IP_R_START) + (eni_index * int(ipa(p.IP_STEP_ENI))) + \
                                      (nsg_index * int(ipa(p.IP_STEP_NSG))) + (acl_index * int(ipa(p.IP_STEP_ACL)))
                        remote_mac_a = str(
                            maca(
                                int(maca(p.MAC_R_START)) +
                                eni_index * int(maca(p.ENI_MAC_STEP)) +
                                nsg_index * int(maca(p.ACL_NSG_MAC_STEP)) +
                                acl_index * int(maca(p.ACL_POLICY_MAC_STEP))
                            )
                        ).replace('-', ':')

                        # Allow
                        for i in range(p.IP_MAPPED_PER_ACL_RULE):  # Per rule prefix
                            remote_expanded_ip = str(remote_ip_a + i * 2)
                            remote_expanded_mac = str(
                                maca(
                                    int(maca(remote_mac_a)) + i * 2
                                )
                            ).replace('-', ':')

                            self.num_yields += 1
                            yield {
                                'DASH_VNET_MAPPING_TABLE:vnet-%d:%s' % (r_vni_id, remote_expanded_ip): {
                                    'routing_type': 'vnet_encap',
                                    'underlay_ip': str(vtep_remote),
                                    'mac_address': remote_expanded_mac,
                                    'use_dst_vni': 'true'
                                },
                                'OP': 'SET'
                            }

            # 3 in 4 enis will have just mapping for gateway ip, for ip that are only routed and not mapped
            else:
                # routed IPs
                print(f'    routed:eni:{eni}', file=sys.stderr)

                remote_expanded_mac = str(
                    maca(
                        int(maca(p.MAC_R_START)) +
                        eni_index * int(maca(p.ENI_MAC_STEP))
                    )
                ).replace('-', ':')
                self.num_yields += 1
                yield {
                    'DASH_VNET_MAPPING_TABLE:vnet-%d:%s' % (r_vni_id, vtep_eni): {
                        'routing_type': 'vnet_encap',
                        'underlay_ip': str(vtep_remote),
                        'mac_address': remote_expanded_mac,
                        'use_dst_vni': 'true'
                    },
                    'OP': 'SET'
                }


if __name__ == '__main__':
    conf = Mappings()
    common_main(conf)
